classdef Line < hgsetget
		
	%%-- Properties --%%
	
	properties (SetAccess=public)
		wl1
		wl2
		t1
		t2
	end
	
	properties (SetAccess=private)
		h=struct('figgui',[],'axes',[],'line',[],'etxt',struct('wl1',[],'wl2',[],'t1',[],'t2',[]));
		opts=struct('etxtformat','%4.3f','nearpix',4,'line',struct('style','-','width',0.5,'color','white','pointer','fleur'),'marker',struct('facecolor','w','edgecolor','w','marker','none','size',6));
	end
	
	properties (Dependent,SetAccess=private)
		coeffs
		wls
		wl_span
		ts
		t_span
		hs
		lims
	end
	
	
	
	%%-- Methods --%%
	
	%% Public %%
	methods
		
		%% Constructor
		function aLine=Line(pt1,pt2,h_figGUI,varargin)
			
			%% Handle lack of arguments
			if(nargin==0)
				
				%% Warning
				warning('trplGUI:Line:NoFigGUIHandle','No figGUI handle provided. Attempting to use callback figure');
				
				%% Try to get limits
				meas=getappdata(gcbf,'meas');
				
				%% Set coordinates top-left corner
				pt1=[max(meas.wl),min(meas.t)];
				pt2=[max(meas.wl),min(meas.t)];
				
				%% Set figGUI handle
				h_figGUI=gcbf;
				
			end
			
			%% Set figGUI handle
			if(isempty(h_figGUI) || ~ishandle(h_figGUI))
				throw(MException('trplGUI:Line:BadFigGUIHandle','Invalid figGUI handle'));
			end
			aLine.h.figgui=h_figGUI;
							
			%% Get figGUI handles structure
			handles=guidata(aLine.h.figgui);

			%% Set default axes handles							% N.b. Cannot set these defaults in properties and require figGUI handles structure
			aLine.h.axes=handles.axMain;

			%% Parse optional parameters
			if(~isempty(varargin))
				[names,values]=varargin2pairs(varargin);
				for ind=1:length(names)
					switch(lower(names{ind}))
						case {'axes','etxtwl1','etxtwl2','etxtt1','etxtt2'}
							aLine.setHandles(names{ind},values{ind});
						case {'etxtformat','nearpix','linestyle','linewidth','linecolor','pointer','marker','markerfacecolor','markeredgecolor','markersize'};
							aLine.setOpts(names{ind},values{ind});
						otherwise
							throw(MException('trplGUI:Line:BadArg','Invalid constructor. Expected valid handle name or option name. Received %s',names{ind}));
					end
				end
			end
			
			%% Set inital and final selection points
			aLine.wl1=pt1(1);
			aLine.t1=pt1(2);
			aLine.wl2=pt2(1);
			aLine.t2=pt2(2);
			
		end
		
		%% Destructor
		function delete(aLine)
			
			%% Clear line
			if(~isempty(aLine.h.line) && ishandle(aLine.h.line))
				delete(aLine.h.line);
			end
			
			%% Reset edit text boxes
			coords={'wl1','wl2','t1','t2'};
			for ind_coord=1:length(coords)
				if(~isempty(aLine.h.etxt.(coords{ind_coord})) && ishandle((aLine.h.etxt.(coords{ind_coord}))))
					set(aLine.h.etxt.(coords{ind_coord}),'String',coords{ind_coord});
				end
			end
			
		end
		
		%% Update
		function update(aLine)
			if(~isempty(aLine.wl1) && ~isempty(aLine.wl2) && ~isempty(aLine.t1) && ~isempty(aLine.t2))		% Catches initial property sets from constructor
				aLine.plot();							% Check for empty handles in these functions
				aLine.setETxt();
			end
		end
		
		%% Assert													% Is this method really useful? If so, should check all properties
		%{
		function tf=assertSel(aLine,varargin)
			
			%% Check coordinate values
			tf.wl=(aLine.assertCoord(aLine.wl1,'wl') && aLine.assertCoord(aLine.wl2,'wl'));
			tf.t=(aLine.assertCoord(aLine.t1,'t') && aLine.assertCoord(aLine.t2,'t'));
			
			%% Check handles
			tf.h=ishandle(aLine.hs);													% Needs rewriting
			
			%% Parse varargin
			if(isempty(varargin))
				cond=(all(tf.wl))&(all(tf.t));
			else
				switch(lower(varargin{1}))
					case 'all'
						cond=(all(tf.wl))&(all(tf.t))&(all(tf.h));
					otherwise
						throw(MException('trplGUI:Line:BadArg','Unrecognised optional argument. Expected "all". Received "%s"',varargin{1}));
				end
			end
			
			%% Set return value
			if(cond)
				tf=true;
			else
				tf=false;
			end
			
		end
		%}
		
		%% Proximity check
		function tf=isNear(aLine,pt)
			
			%% Get axes properties
			ax.lims=struct('wl',get(aLine.h.axes,'XLim'),'t',get(aLine.h.axes,'YLim'));
			ax.units=get(aLine.h.axes,'Units');
			set(aLine.h.axes,'Units','pixels');
			axpos=get(aLine.h.axes,'Position');
			set(aLine.h.axes,'Units',ax.units);
			ax.dims=struct('wl',axpos(3),'t',axpos(4));
			
			%% Define proximity
			near.pix=aLine.opts.nearpix;
			near.units=struct('wl',near.pix*abs(diff(ax.lims.wl))/ax.dims.wl,'t',near.pix*abs(diff(ax.lims.t))/ax.dims.t);
			
			%% Define default return value
			tf=false;
			
			%% Check proximity
			if(isempty(aLine.coeffs) || any(isinf(aLine.coeffs)))
				if(pt(1)>=(aLine.wl1-near.units.wl) && pt(1)<=(aLine.wl1+near.units.wl))
					tf=true;
				end
			else
				t_line=((aLine.coeffs(2)).*pt(1))+aLine.coeffs(1);
				if(pt(2)>=(t_line-near.units.t) && pt(2)<=(t_line+near.units.t))
					tf=true;
				end
			end
			
		end
		
		%% Assert coordinate value
		function [tf,bound]=assertCoord(aLine,coord,axis)
			
			%% Validate axis
			switch(lower(axis))
				case {'wl','t'}
				otherwise
					throw(MException('trplGUI:Line:BadAxis','Axis not recognised. Expected "wl" or "t". Receieved "%s"',axis));
			end
			axis=lower(axis);
			
			%% Catch vector values
			if(~isscalar(coord))
				throw(MException('trplGUI:Line:VecCoord','Selection coordinate must be scalar. Attempted to set coordinate to a vector of length %d',length(coord)));
			end
			
			%% Catch non-numeric coordinates
			if(~isnumeric(coord))
				throw(MException('trplGUI:Line:NonNumCoord','Selection coordinate must be numeric. Attempted to set coordinate to a non-numeric value'));
			end
			
			%% Catch NaN and Inf coordinates
			if(any(~isfinite(coord)))
				throw(MException('trplGUI:Line:NonFiniteCoord','Selection coordinate must be finite. Attempted to set coordinate to NaN or Inf'));
			end
			
			%% Check coordinates within measurements limits
			if(coord<aLine.lims.(axis)(1))
				tf=false;
				bound=aLine.lims.(axis)(1);
			elseif(coord>aLine.lims.(axis)(2))
				tf=false;
				bound=aLine.lims.(axis)(2);
			else
				tf=true;
				bound=[];
			end
			
		end
		
		%% Set options
		function setOpts(aLine,varargin)
			
			%% Parse varargin
			[opts,values]=varargin2pairs(varargin);																									%#ok<PROP>
			for ind=1:length(opts)																													%#ok<PROP>
				switch(lower(opts{ind}))																											%#ok<PROP>
					case 'nearpix'
						if(isnumeric(values{ind}))
							aLine.opts.nearpix=values{ind};
						else
							warning('trplGUI:Line:BadNearPix','Non-numeric value of NearPix option. Setting to default value (%d)',aLine.opts.nearpix);
						end
					case 'etxtformat'
						aLine.opts.etxtformat=values{ind};
					case 'linestyle'
						aLine.opts.line.style=values{ind};
					case 'linewidth'
						aLine.opts.line.width=values{ind};
					case 'linecolor'
						aLine.opts.line.color=values{ind};
					case 'pointer'
						aLine.opts.line.pointer=values{ind};
					case 'marker'
						aLine.opts.marker.marker=values{ind};
					case 'markerfacecolor'
						aLine.opts.marker.facecolor=values{ind};
					case 'markeredgecolor'
						aLine.opts.marker.edgecolor=values{ind};
					case 'markersize'
						aLine.opts.marker.size=values{ind};
					otherwise
						warning('trplGUI:Line:BadOptionName','Invalid option. Expected "NearPix", "ETxtFormat", "LineStyle", "LineWidth", "LineColor", "Marker", "MarkerEdgeColor", "MarkerFaceColor", "MarkerSize". Received %s. Ignoring OptionName-OptionValue pair',opts{ind}); %#ok<PROP>
				end
			end
			
			%% Update
			aLine.update();
			
		end
		
		%% Set handles
		function setHandles(aLine,varargin)
			
			%% Parse varargin
			[names,values]=varargin2pairs(varargin);
			names=lower(names);
			for ind=1:length(names)
				switch(names{ind})
					case 'axes'
						if(isempty(values{ind}) || ishandle(values{ind}))
							aLine.h.axes=values{ind};
						else
							warning('trplGUI:Line:BadAxesHandle','Invalid axes handle for line. Setting h.axes to []');
							aLine.h.axes=[];
						end
					case 'etxtwl1'
						if(isempty(values{ind}) || ishandle(values{ind}))
							aLine.h.etxt.wl1=values{ind};
						else
							warning('trplGUI:Line:BadWl1EtxtHandle','Invalid edit text handle for wl1. Setting h.etxt.wl1 to []');
							aLine.h.etxt.wl1=[];
						end
					case 'etxtwl2'
						if(isempty(values{ind}) || ishandle(values{ind}))
							aLine.h.etxt.wl2=values{ind};
						else
							warning('trplGUI:Line:BadWl2EtxtHandle','Invalid edit text handle for wl2. Setting h.etxt.wl2 to []');
							aLine.h.etxt.wl2=[];
						end
					case 'etxtt1'
						if(isempty(values{ind}) || ishandle(values{ind}))
							aLine.h.etxt.t1=values{ind};
						else
							warning('trplGUI:Line:BadT1EtxtHandle','Invalid edit text handle for t1. Setting h.etxt.t1 to []');
							aLine.h.etxt.t1=[];
						end
					case 'etxtt2'
						if(isempty(values{ind}) || ishandle(values{ind}))
							aLine.h.etxt.t2=values{ind};
						else
							warning('trplGUI:Line:BadT2EtxtHandle','Invalid edit text handle for t2. Setting h.etxt.t2 to []');
							aLine.h.etxt.t2=[];
						end
					otherwise
						warning('trplGUI:Line:BadHandleName','Invalid handle name. Expected "Axes", "ETxtWl1", "ETxtWl2", "ETxtT1", or "ETxtT2". Received %s. Ignoring HandleName-HandleValue pair',names{ind});
				end
			end
			
			%% Update
			aLine.update();
			
		end
		
	end
	
	%% Private %%
	methods
		
		%% Plot line
		function plot(aLine)
			
			if(~isempty(aLine.h.axes) && ishandle(aLine.h.axes))
				
				%% Plot new line
				if(any(isempty(aLine.h.line)) || any(~ishandle(aLine.h.line)))
					
					%% Plot line
					aLine.h.line=line('Parent',aLine.h.axes,'XData',[aLine.wl1,aLine.wl2],'YData',[aLine.t1,aLine.t2],'Color',aLine.opts.line.color,'LineStyle',aLine.opts.line.style,'LineWidth',aLine.opts.line.width,'Marker',aLine.opts.marker.marker,'MarkerEdgeColor',aLine.opts.marker.edgecolor,'MarkerFaceColor',aLine.opts.marker.facecolor,'MarkerSize',aLine.opts.marker.size);
					
					%% Set line pointer
					iptSetPointerBehavior(aLine.h.line,@(figHandle,currentPoint) setptr(figHandle,aLine.opts.line.pointer));
										
					%% Enable pointer manager
					iptPointerManager(get(aLine.h.axes,'Parent'),'enable');
					
					
				%% Modify existing line
				else
					
					%% Modify lines
					set(aLine.h.line,'Parent',aLine.h.axes,'XData',[aLine.wl1,aLine.wl2],'YData',[aLine.t1,aLine.t2],'Color',aLine.opts.line.color,'LineStyle',aLine.opts.line.style,'LineWidth',aLine.opts.line.width,'Marker',aLine.opts.marker.marker,'MarkerEdgeColor',aLine.opts.marker.edgecolor,'MarkerFaceColor',aLine.opts.marker.facecolor,'MarkerSize',aLine.opts.marker.size);

					%% Set line pointers							% These are a bit heavy and unnecessary
					%{
					iptSetPointerBehaviour(aLine.h.line,@(figHandle,currentPoint) setptr(figHandle,aLine.opts.line.pointer));
					%}

				end
				
			end
			
		end	
				
		%% Set edit text boxes
		function setETxt(aLine)
			
			%% Set etxt boxes
			if(~isempty(aLine.h.etxt.wl1) && ishandle(aLine.h.etxt.wl1))
				set(aLine.h.etxt.wl1,'String',num2str(aLine.wl1,aLine.opts.etxtformat));
			end
			if(~isempty(aLine.h.etxt.wl2) && ishandle(aLine.h.etxt.wl2))
				set(aLine.h.etxt.wl2,'String',num2str(aLine.wl2,aLine.opts.etxtformat));
			end
			if(~isempty(aLine.h.etxt.t1) && ishandle(aLine.h.etxt.t1))
				set(aLine.h.etxt.t1,'String',num2str(aLine.t1,aLine.opts.etxtformat));
			end
			if(~isempty(aLine.h.etxt.t2) && ishandle(aLine.h.etxt.t2))
				set(aLine.h.etxt.t2,'String',num2str(aLine.t2,aLine.opts.etxtformat));
			end
		
		end
		
	end
	
	%% Access %%
	methods	
		
		%% Set: wl1
		function aLine=set.wl1(aLine,wl)
			
			%% Validate value
			[ret,bound]=aLine.assertCoord(wl,'wl');
			if(~ret)
				warning('trplGUI:Line:BadWl1','Wavelength value outside measurement bound. Expected values between %g and %g. Received %g. Setting wl1 to bound (%g)',aLine.lims.wl(1),aLine.lims.wl(2),wl,bound); %#ok<MCSUP>
				wl=bound;
			end
			
			%% Set value
			aLine.wl1=wl;
			aLine.update();
			
		end
		
		%% Set: wl2
		function aLine=set.wl2(aLine,wl)
			
			%% Validate value
			[ret,bound]=aLine.assertCoord(wl,'wl');
			if(~ret)
				warning('trplGUI:Line:BadWl2','Wavelength value outside measurement bound. Expected values between %g and %g. Received %g. Setting wl2 to bound (%g)',aLine.lims.wl(1),aLine.lims.wl(2),wl,bound); %#ok<MCSUP>
				wl=bound;
			end
			
			%% Set value
			aLine.wl2=wl;
			aLine.update();
			
		end
		
		%% Set: t1
		function aLine=set.t1(aLine,t)
				
			%% Validate value
			[ret,bound]=aLine.assertCoord(t,'t');
			if(~ret)
				warning('trplGUI:Line:BadT1','Time value outside measurement bound. Expected values between %g and %g. Received %g. Setting t1 to bound (%g)',aLine.lims.t(1),aLine.lims.t(2),t,bound); %#ok<MCSUP>
				t=bound;
			end
			
			%% Set value
			aLine.t1=t;
			aLine.update();
			
		end
		
		%% Set: t2
		function aLine=set.t2(aLine,t)
			
			%% Validate value
			[ret,bound]=aLine.assertCoord(t,'t');
			if(~ret)
				warning('trplGUI:Line:BadT2','Time value outside measurement bound. Expected values between %g and %g. Received %g. Setting t2 to bound (%g)',aLine.lims.t(1),aLine.lims.t(2),t,bound); %#ok<MCSUP>
				t=bound;
			end
			
			%% Set value
			aLine.t2=t;
			aLine.update();
		
		end
		
		%% Get: coeffs
		function coeffs=get.coeffs(aLine)
			
			%% Disable nearly singular matrix warning
			warn_sing=warning('off','MATLAB:nearlySingularMatrix');
			
			%% Clear last warning
			lastwarn('');
			
			%% Attempt coefficient calculation
			coeffs=([ones(1,2);aLine.wls]')\(aLine.ts');
			
			%% Check for nearly singular matrix warning
			[warn.msg,warn.id]=lastwarn;
			if(strcmp(warn.id,'MATLAB:nearlySingularMatrix'))
				coeffs=[Inf,Inf];
			end
			
			%% Restore nearly singular matrix warning
			warning(warn_sing.state,warn_sing.identifier);
			
		end
		
		%% Get: wls
		function wls=get.wls(aLine)
			wls=[aLine.wl1,aLine.wl2];
		end
		
		%% Get wl_span
		function wl_span=get.wl_span(aLine)
			wl_span=abs(diff(aLine.wls));
		end
		
		%% Get: ts
		function ts=get.ts(aLine)
			ts=[aLine.t1,aLine.t2];
		end
		
		%% Get t_span
		function t_span=get.t_span(aLine)
			t_span=abs(diff(aLine.ts));
		end
		
		%% Get: hs
		function hs=get.hs(aLine)
			hs=struct('line',aLine.h.line,'etxt',[aLine.h.etxt.wl1;aLine.h.etxt.wl2;aLine.h.etxt.t1;aLine.h.etxt.t2]);
		end
		
		%% Get: lims
		function lims=get.lims(aLine)
			meas=getappdata(aLine.h.figgui,'meas');
			lims=struct('wl',[min(meas.wl),max(meas.wl)],'t',[min(meas.t),max(meas.t)]);
		end
		
	end
		
end