classdef Parab < hgsetget
	
	%%-- Properties --%%
	
	properties (SetAccess=public)
		wl1
		wl2
		wl3
		t1
		t2
		t3
	end
	
	properties (SetAccess=private)
		h=struct('figgui',[],'axes',[],'parab',struct('line',[],'pt1',[],'pt2',[],'pt3',[]),'etxt',struct('wl1',[],'wl2',[],'wl3',[],'t1',[],'t2',[],'t3',[]));
		opts=struct('etxtformat','%4.3f','nearpix',4,'line',struct('style','-','width',0.5,'color','white','pointer','fleur'),'pt',struct('marker','s','style','none','facecolor','w','edgecolor','w','size',6,'pointer','fleur'));
	end
	
	properties (Dependent,SetAccess=private)
		coeffs
		wls
		ts
		hs
		lims
	end
	
	
	
	%%-- Methods --%%
	
	%% Public %%
	methods
		
		%% Constructor
		function aParab=Parab(pt1,pt2,pt3,h_figGUI,varargin)
			
			%% Handle lack of arguments
			if(nargin==0)
				
				%% Warning
				warning('trplGUI:Parab:NoFigGUIHandle','No figGUI handle provided. Attempting to use callback figure');
				
				%% Try to get limits
				meas=getappdata(gcbf,'meas');
				
				%% Set default coordinates
				pt1=[max(meas.wl),min(meas.t)];
				pt2=[mean([max(meas.wl),min(meas.wl)]),min(meas.t)];
				pt3=[min(meas.wl),min(meas.t)];
				
				%% Set figGUI handle
				h_figGUI=gcbf;
				
			end
			
			%% Set figGUI handle
			if(isempty(h_figGUI) || ~ishandle(h_figGUI))
				throw(MException('trplGUI:Parab:BadFigGUIHandle','Invalid figGUI handle'));
			end
			aParab.h.figgui=h_figGUI;
							
			%% Get figGUI handles structure
			handles=guidata(aParab.h.figgui);

			%% Set default axes handles							% N.b. Cannot set these defaults in properties and require figGUI handles structure
			aParab.h.axes=handles.axMain;

			%% Parse optional parameters
			if(~isempty(varargin))
				[names,values]=varargin2pairs(varargin);
				for ind=1:length(names)
					switch(lower(names{ind}))
						case {'axes','etxtwl1','etxtwl2','etxtwl3','etxtt1','etxtt2','etxtt3'}
							aParab.setHandles(names{ind},values{ind});
						case {'etxtformat','nearpix','linestyle','linewidth','linecolor','linepointer','ptmarker','ptfacecolor','ptedgecolor','ptsize','ptpointer'};
							aParab.setOpts(names{ind},values{ind});
						otherwise
							throw(MException('trplGUI:Parab:BadArg','Invalid constructor. Expected valid handle name or option name. Received %s',names{ind}));
					end
				end
			end
			
			%% Set inital and final selection points
			aParab.wl1=pt1(1);
			aParab.t1=pt1(2);
			aParab.wl2=pt2(1);
			aParab.t2=pt2(2);
			aParab.wl3=pt3(1);
			aParab.t3=pt3(2);
			
		end
		
		%% Destructor
		function delete(aParab)
			
			%% Clear parabola
			h_parab=[aParab.hs.line;aParab.hs.pt];
			if(~isempty(h_parab))
				delete(h_parab(ishandle(h_parab)));
			end
			
			%% Reset edit text boxes
			coords={'wl1','wl2','wl3','t1','t2','t3'};
			for ind_coord=1:length(coords)
				if(~isempty(aParab.h.etxt.(coords{ind_coord})) && ishandle((aParab.h.etxt.(coords{ind_coord}))))
					set(aParab.h.etxt.(coords{ind_coord}),'String',coords{ind_coord});
				end
			end
			
		end

		%% Update
		function update(aParab)
			if(length([aParab.wls,aParab.ts])==6)																% Catches initial property sets from constructor
				aParab.plot();																					% Check for empty handles in these functions
				aParab.setETxt();
			end
		end
		
		%% Assert													% Is this method really useful? If so, should check all properties
		%{
		function tf=assertSel(aParab,varargin)
			
			%% Check coordinate values
			tf.wl=(aParab.assertCoord(aParab.wl1,'wl') && aParab.assertCoord(aParab.wl2,'wl'));
			tf.t=(aParab.assertCoord(aParab.t1,'t') && aParab.assertCoord(aParab.t2,'t'));
			
			%% Check handles
			tf.h=ishandle(aParab.hs);													% Needs rewriting
			
			%% Parse varargin
			if(isempty(varargin))
				cond=(all(tf.wl))&(all(tf.t));
			else
				switch(lower(varargin{1}))
					case 'all'
						cond=(all(tf.wl))&(all(tf.t))&(all(tf.h));
					otherwise
						throw(MException('trplGUI:Parab:BadArg','Unrecognised optional argument. Expected "all". Received "%s"',varargin{1}));
				end
			end
			
			%% Set return value
			if(cond)
				tf=true;
			else
				tf=false;
			end
			
		end
		%}
		
		%% Proximity check
		function tf=isNear(aParab,pt)
			
			%% Get axes properties
			ax.lims=struct('wl',get(aParab.h.axes,'XLim'),'t',get(aParab.h.axes,'YLim'));
			ax.units=get(aParab.h.axes,'Units');
			set(aParab.h.axes,'Units','pixels');
			axpos=get(aParab.h.axes,'Position');
			set(aParab.h.axes,'Units',ax.units);
			ax.dims=struct('wl',axpos(3),'t',axpos(4));
			
			%% Define proximity
			near.pix=aParab.opts.nearpix;
			near.units=struct('wl',near.pix*abs(diff(ax.lims.wl))/ax.dims.wl,'t',near.pix*abs(diff(ax.lims.t))/ax.dims.t);
			
			%% Define default return value
			tf=false(1,4);														% tf=[line,pt1,pt2,pt3]
			
			%% Check proximity to line
			if(~isempty(aParab.coeffs))
				t_line=[1,pt(1),pt(1)^2]*aParab.coeffs;
				if(pt(2)>=t_line-near.units.t && pt(2)<=t_line+near.units.t)
					tf(1)=true;
				end
			end
			
			%% Check proximity to points
			if(pt(1)>=aParab.wl1-near.units.wl && pt(1)<=aParab.wl1+near.units.wl && pt(2)>=aParab.t1-near.units.t && pt(2)<=aParab.t1+near.units.t)
				tf(2)=true;
			end
			if(pt(1)>=aParab.wl2-near.units.wl && pt(1)<=aParab.wl2+near.units.wl && pt(2)>=aParab.t2-near.units.t && pt(2)<=aParab.t2+near.units.t)
				tf(3)=true;
			end
			if(pt(1)>=aParab.wl3-near.units.wl && pt(1)<=aParab.wl3+near.units.wl && pt(2)>=aParab.t3-near.units.t && pt(2)<=aParab.t3+near.units.t)
				tf(4)=true;
			end
			
		end
		
		%% Assert coordinate value
		function [tf,bound]=assertCoord(aParab,coord,axis)
			
			%% Validate axis
			switch(lower(axis))
				case {'wl','t'}
				otherwise
					throw(MException('trplGUI:Parab:BadAxis','Axis not recognised. Expected "wl" or "t". Receieved "%s"',axis));
			end
			axis=lower(axis);
			
			%% Catch vector values
			if(~isscalar(coord))
				throw(MException('trplGUI:Parab:VecCoord','Selection coordinate must be scalar. Attempted to set coordinate to a vector of length %d',length(coord)));
			end
			
			%% Catch non-numeric coordinates
			if(~isnumeric(coord))
				throw(MException('trplGUI:Parab:NonNumCoord','Selection coordinate must be numeric. Attempted to set coordinate to a non-numeric value'));
			end
			
			%% Catch NaN and Inf coordinates
			if(any(~isfinite(coord)))
				throw(MException('trplGUI:Parab:NonFiniteCoord','Selection coordinate must be finite. Attempted to set coordinate to NaN or Inf'));
			end
			
			%% Check coordinates within measurements limits
			if(coord<aParab.lims.(axis)(1))
				tf=false;
				bound=aParab.lims.(axis)(1);
			elseif(coord>aParab.lims.(axis)(2))
				tf=false;
				bound=aParab.lims.(axis)(2);
			else
				tf=true;
				bound=[];
			end
			
		end
		
		%% Set options
		function setOpts(aParab,varargin)
			
			%% Parse varargin
			[opts,values]=varargin2pairs(varargin);																									%#ok<PROP>
			for ind=1:length(opts)																													%#ok<PROP>
				switch(lower(opts{ind}))																											%#ok<PROP>
					case 'nearpix'
						if(isnumeric(values{ind}))
							aParab.opts.nearpix=values{ind};
						else
							warning('trplGUI:Parab:BadNearPix','Non-numeric value of NearPix option. Setting to default value (%d)',aParab.opts.nearpix);
						end
					case 'etxtformat'
						aParab.opts.etxtformat=values{ind};
					case 'linestyle'
						aParab.opts.line.style=values{ind};
					case 'linewidth'
						aParab.opts.line.width=values{ind};
					case 'linecolor'
						aParab.opts.line.color=values{ind};
					case 'linepointer'
						aParab.opts.line.pointer=values{ind};
					case 'ptmarker'
						aParab.opts.pt.marker=values{ind};
					case 'ptfacecolor'
						aParab.opts.pt.facecolor=values{ind};
					case 'ptedgecolor'
						aParab.opts.pt.edgecolor=values{ind};
					case 'ptsize'
						aParab.opts.pt.size=values{ind};
					case 'ptpointer'
						aParab.opts.pt.pointer=values{ind};
					otherwise
						warning('trplGUI:Parab:BadOptionName','Invalid option. Expected "NearPix", "ETxtFormat", "LineStyle", "LineWidth", "LineColor", "LinePointer", "PtMarker", "PtFaceColor", "PtEdgeColor", "PtSize", "PtPointer". Received %s. Ignoring OptionName-OptionValue pair',opts{ind}); %#ok<PROP>
				end
			end
			
			%% Update
			aParab.update();
			
		end
		
		%% Set handles
		function setHandles(aParab,varargin)
			
			%% Parse varargin
			[names,values]=varargin2pairs(varargin);
			names=lower(names);
			for ind=1:length(names)
				switch(names{ind})
					case 'axes'
						if(isempty(values{ind}) || ishandle(values{ind}))
							aParab.h.axes=values{ind};
						else
							warning('trplGUI:Parab:BadAxesHandle','Invalid axes handle for parabola. Setting h.axes to []');
							aParab.h.axes=[];
						end
					case 'etxtwl1'
						if(isempty(values{ind}) || ishandle(values{ind}))
							aParab.h.etxt.wl1=values{ind};
						else
							warning('trplGUI:Parab:BadWl1EtxtHandle','Invalid edit text handle for wl1. Setting h.etxt.wl1 to []');
							aParab.h.etxt.wl1=[];
						end
					case 'etxtwl2'
						if(isempty(values{ind}) || ishandle(values{ind}))
							aParab.h.etxt.wl2=values{ind};
						else
							warning('trplGUI:Parab:BadWl2EtxtHandle','Invalid edit text handle for wl2. Setting h.etxt.wl2 to []');
							aParab.h.etxt.wl2=[];
						end
					case 'etxtwl3'
						if(isempty(values{ind}) || ishandle(values{ind}))
							aParab.h.etxt.wl3=values{ind};
						else
							warning('trplGUI:Parab:BadWl3EtxtHandle','Invalid edit text handle for wl3. Setting h.etxt.wl3 to []');
							aParab.h.etxt.wl3=[];
						end
					case 'etxtt1'
						if(isempty(values{ind}) || ishandle(values{ind}))
							aParab.h.etxt.t1=values{ind};
						else
							warning('trplGUI:Parab:BadT1EtxtHandle','Invalid edit text handle for t1. Setting h.etxt.t1 to []');
							aParab.h.etxt.t1=[];
						end
					case 'etxtt2'
						if(isempty(values{ind}) || ishandle(values{ind}))
							aParab.h.etxt.t2=values{ind};
						else
							warning('trplGUI:Parab:BadT2EtxtHandle','Invalid edit text handle for t2. Setting h.etxt.t2 to []');
							aParab.h.etxt.t2=[];
						end
					case 'etxtt3'
						if(isempty(values{ind}) || ishandle(values{ind}))
							aParab.h.etxt.t3=values{ind};
						else
							warning('trplGUI:Parab:BadT3EtxtHandle','Invalid edit text handle for t3. Setting h.etxt.t3 to []');
							aParab.h.etxt.t3=[];
						end
					otherwise
						warning('trplGUI:Parab:BadHandleName','Invalid handle name. Expected "Axes", "ETxtWl1", "ETxtWl2", "ETxtWl3", "ETxtT1", "ETxtT2", or "ETxtT3". Received %s. Ignoring HandleName-HandleValue pair',names{ind});
				end
			end
			
			%% Update
			aParab.update();
			
		end
		
	end
	
	%% Private %%
	methods
		
		%% Plot parabola
		function plot(aParab)
			
			if(~isempty(aParab.h.axes) && ishandle(aParab.h.axes))
				
				%% Get measurement appdata
				meas=getappdata(aParab.h.figgui,'meas');
				
				%% No coefficients - do not plot (and remove previous parabola line)
				if(isempty(aParab.coeffs))
				
					if(~isempty(aParab.hs.line) && ishandle(aParab.hs.line))
						delete(aParab.hs.line);
					end
					
				%% Plot parabola line
				else
					
					if(isempty(aParab.hs.line) || ~ishandle(aParab.hs.line))

						%% Plot new parabola line
						aParab.h.parab.line=line('Parent',aParab.h.axes,'XData',meas.wl,'YData',([ones(length(meas.wl),1),meas.wl,(meas.wl).^2])*aParab.coeffs,'Color',aParab.opts.line.color,'LineStyle',aParab.opts.line.style,'LineWidth',aParab.opts.line.width);

						%% Set line pointer
						iptSetPointerBehavior(aParab.h.parab.line,@(figHandle,currentPoint) setptr(figHandle,aParab.opts.line.pointer));

						%% Enable pointer manager
						iptPointerManager(get(aParab.h.axes,'Parent'),'enable');

					else

						%% Modify existing parabola line
						set(aParab.h.parab.line,'Parent',aParab.h.axes,'XData',meas.wl,'YData',([ones(length(meas.wl),1),meas.wl,(meas.wl).^2])*aParab.coeffs,'Color',aParab.opts.line.color,'LineStyle',aParab.opts.line.style,'LineWidth',aParab.opts.line.width);
						
					end

				end
				
				%% Plot parabola points
				if(isempty(aParab.hs.pt) || any(~ishandle(aParab.hs.pt)))
					
					%% Delete any valid points
					delete(aParab.hs.pt(ishandle(aParab.hs.pt)));
					
					%% Plot new points
					aParab.h.parab.pt1=line('Parent',aParab.h.axes,'XData',aParab.wl1,'YData',aParab.t1,'LineStyle',aParab.opts.pt.style,'Marker',aParab.opts.pt.marker,'MarkerSize',aParab.opts.pt.size,'MarkerFaceColor',aParab.opts.pt.facecolor,'MarkerEdgeColor',aParab.opts.pt.edgecolor);
					aParab.h.parab.pt2=line('Parent',aParab.h.axes,'XData',aParab.wl2,'YData',aParab.t2,'LineStyle',aParab.opts.pt.style,'Marker',aParab.opts.pt.marker,'MarkerSize',aParab.opts.pt.size,'MarkerFaceColor',aParab.opts.pt.facecolor,'MarkerEdgeColor',aParab.opts.pt.edgecolor);
					aParab.h.parab.pt3=line('Parent',aParab.h.axes,'XData',aParab.wl3,'YData',aParab.t3,'LineStyle',aParab.opts.pt.style,'Marker',aParab.opts.pt.marker,'MarkerSize',aParab.opts.pt.size,'MarkerFaceColor',aParab.opts.pt.facecolor,'MarkerEdgeColor',aParab.opts.pt.edgecolor);
					
					%% Set points pointer
					iptSetPointerBehavior(aParab.h.parab.pt1,@(figHandle,currentPoint) setptr(figHandle,aParab.opts.pt.pointer));
					iptSetPointerBehavior(aParab.h.parab.pt2,@(figHandle,currentPoint) setptr(figHandle,aParab.opts.pt.pointer));
					iptSetPointerBehavior(aParab.h.parab.pt3,@(figHandle,currentPoint) setptr(figHandle,aParab.opts.pt.pointer));
					
					%% Enable pointer manager
					iptPointerManager(get(aParab.h.axes,'Parent'),'enable');
					
				else
					
					%% Modify existing points
					set(aParab.h.parab.pt1,'Parent',aParab.h.axes,'XData',aParab.wl1,'YData',aParab.t1,'LineStyle',aParab.opts.pt.style,'Marker',aParab.opts.pt.marker,'MarkerSize',aParab.opts.pt.size,'MarkerFaceColor',aParab.opts.pt.facecolor,'MarkerEdgeColor',aParab.opts.pt.edgecolor);
					set(aParab.h.parab.pt2,'Parent',aParab.h.axes,'XData',aParab.wl2,'YData',aParab.t2,'LineStyle',aParab.opts.pt.style,'Marker',aParab.opts.pt.marker,'MarkerSize',aParab.opts.pt.size,'MarkerFaceColor',aParab.opts.pt.facecolor,'MarkerEdgeColor',aParab.opts.pt.edgecolor);
					set(aParab.h.parab.pt3,'Parent',aParab.h.axes,'XData',aParab.wl3,'YData',aParab.t3,'LineStyle',aParab.opts.pt.style,'Marker',aParab.opts.pt.marker,'MarkerSize',aParab.opts.pt.size,'MarkerFaceColor',aParab.opts.pt.facecolor,'MarkerEdgeColor',aParab.opts.pt.edgecolor);
					
				end
				
			end
			
		end	
				
		%% Set edit text boxes
		function setETxt(aParab)
			
			%% Set etxt boxes
			if(~isempty(aParab.h.etxt.wl1) && ishandle(aParab.h.etxt.wl1))
				set(aParab.h.etxt.wl1,'String',num2str(aParab.wl1,aParab.opts.etxtformat));
			end
			if(~isempty(aParab.h.etxt.wl2) && ishandle(aParab.h.etxt.wl2))
				set(aParab.h.etxt.wl2,'String',num2str(aParab.wl2,aParab.opts.etxtformat));
			end
			if(~isempty(aParab.h.etxt.wl3) && ishandle(aParab.h.etxt.wl3))
				set(aParab.h.etxt.wl3,'String',num2str(aParab.wl3,aParab.opts.etxtformat));
			end
			if(~isempty(aParab.h.etxt.t1) && ishandle(aParab.h.etxt.t1))
				set(aParab.h.etxt.t1,'String',num2str(aParab.t1,aParab.opts.etxtformat));
			end
			if(~isempty(aParab.h.etxt.t2) && ishandle(aParab.h.etxt.t2))
				set(aParab.h.etxt.t2,'String',num2str(aParab.t2,aParab.opts.etxtformat));
			end
			if(~isempty(aParab.h.etxt.t3) && ishandle(aParab.h.etxt.t3))
				set(aParab.h.etxt.t3,'String',num2str(aParab.t3,aParab.opts.etxtformat));
			end
		
		end
		
	end
	
	
	%% Access %%
	methods	
		
		%% Set: wl1
		function aParab=set.wl1(aParab,wl)
			
			%% Validate value
			[ret,bound]=aParab.assertCoord(wl,'wl');
			if(~ret)
				warning('trplGUI:Parab:BadWl1','Wavelength value outside measurement bound. Expected values between %g and %g. Received %g. Setting wl1 to bound (%g)',aParab.lims.wl(1),aParab.lims.wl(2),wl,bound); %#ok<MCSUP>
				wl=bound;
			end
			
			%% Set value
			aParab.wl1=wl;
			aParab.update();
			
		end
		
		%% Set: wl2
		function aParab=set.wl2(aParab,wl)
			
			%% Validate value
			[ret,bound]=aParab.assertCoord(wl,'wl');
			if(~ret)
				warning('trplGUI:Parab:BadWl2','Wavelength value outside measurement bound. Expected values between %g and %g. Received %g. Setting wl2 to bound (%g)',aParab.lims.wl(1),aParab.lims.wl(2),wl,bound); %#ok<MCSUP>
				wl=bound;
			end
			
			%% Set value
			aParab.wl2=wl;
			aParab.update();
			
		end
		
		%% Set: wl3
		function aParab=set.wl3(aParab,wl)
			
			%% Validate value
			[ret,bound]=aParab.assertCoord(wl,'wl');
			if(~ret)
				warning('trplGUI:Parab:BadWl3','Wavelength value outside measurement bound. Expected values between %g and %g. Received %g. Setting wl3 to bound (%g)',aParab.lims.wl(1),aParab.lims.wl(2),wl,bound); %#ok<MCSUP>
				wl=bound;
			end
			
			%% Set value
			aParab.wl3=wl;
			aParab.update();
			
		end
		
		%% Set: t1
		function aParab=set.t1(aParab,t)
				
			%% Validate value
			[ret,bound]=aParab.assertCoord(t,'t');
			if(~ret)
				warning('trplGUI:Parab:BadT1','Time value outside measurement bound. Expected values between %g and %g. Received %g. Setting t1 to bound (%g)',aParab.lims.t(1),aParab.lims.t(2),t,bound); %#ok<MCSUP>
				t=bound;
			end
			
			%% Set value
			aParab.t1=t;
			aParab.update();
			
		end
		
		%% Set: t2
		function aParab=set.t2(aParab,t)
			
			%% Validate value
			[ret,bound]=aParab.assertCoord(t,'t');
			if(~ret)
				warning('trplGUI:Parab:BadT2','Time value outside measurement bound. Expected values between %g and %g. Received %g. Setting t2 to bound (%g)',aParab.lims.t(1),aParab.lims.t(2),t,bound); %#ok<MCSUP>
				t=bound;
			end
			
			%% Set value
			aParab.t2=t;
			aParab.update();
		
		end
		
		%% Set: t3
		function aParab=set.t3(aParab,t)
			
			%% Validate value
			[ret,bound]=aParab.assertCoord(t,'t');
			if(~ret)
				warning('trplGUI:Parab:BadT3','Time value outside measurement bound. Expected values between %g and %g. Received %g. Setting t2 to bound (%g)',aParab.lims.t(1),aParab.lims.t(2),t,bound); %#ok<MCSUP>
				t=bound;
			end
			
			%% Set value
			aParab.t3=t;
			aParab.update();
		
		end
		
		%% Get: coeffs
		function coeffs=get.coeffs(aParab)
			
			%% Disable nearly singular matrix warning
			warn_sing=warning('off','MATLAB:nearlySingularMatrix');
			
			%% Clear last warning
			lastwarn('');
			
			%% Attempt coefficient calculation
			coeffs=([ones(1,3);aParab.wls;(aParab.wls).^2]')\(aParab.ts');
			
			%% Check for nearly singular matrix warning
			[warn.msg,warn.id]=lastwarn;
			if(strcmp(warn.id,'MATLAB:nearlySingularMatrix'))
				coeffs=[];
			end
			
			%% Restore nearly singular matrix warning
			warning(warn_sing.state,warn_sing.identifier);
			
		end
		
		%% Get: wls
		function wls=get.wls(aParab)
			wls=[aParab.wl1,aParab.wl2,aParab.wl3];
		end
		
		%% Get: ts
		function ts=get.ts(aParab)
			ts=[aParab.t1,aParab.t2,aParab.t3];
		end
				
		%% Get: hs
		function hs=get.hs(aParab)
			hs=struct('line',aParab.h.parab.line,'pt',[aParab.h.parab.pt1;aParab.h.parab.pt2;aParab.h.parab.pt3],'etxt',[aParab.h.etxt.wl1;aParab.h.etxt.wl2;aParab.h.etxt.wl3;aParab.h.etxt.t1;aParab.h.etxt.t2;aParab.h.etxt.t3]);
		end
		
		%% Get: lims
		function lims=get.lims(aParab)
			meas=getappdata(aParab.h.figgui,'meas');
			lims=struct('wl',[min(meas.wl),max(meas.wl)],'t',[min(meas.t),max(meas.t)]);
		end
		
	end
	
end