%% Name:		trplBGSub.m
%% Description:	Subtract background from Time Resolved PL data obtained in the Chameleon lab
%% Authors:		M.J.P Alcocer [marcelo.alcocer@iit.it || marcelo.j.p.alcocer@gmail.com]
%% Licensing:	Copyright  2011  M.J.P Alcocer
%%				This file is part of tprl. trpl is free software: you can redistribute it and/or modify it under the terms of the GNU
%%				General Public License as published by the Free Software Foundation, either version 3 of the License, or any later version.
%%				trpl is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of 
%%				MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. You should have
%%				received a copy of the GNU General Public License along with trpl. If not, see <http://www.gnu.org/licenses/>.

function data = trplBGSub(data)
	
	%% Syntax:
	%%
	%%		data = trplBGSub(data)
	%%
	%% Description:
	%%
	%%		data = trplBGSub(data) corrects Chameleon TRPL data for pre-rise background noise. Correction is performed by subtracting the
	%%		mean pre-rise signal at each wavelength. Currently, the pre-rise period is simply defined as the first 5% of the temporal range.
	%%		This will be improved in future versions.
	%%
	%%			Arguments:
	%%				data:	2D array of TRPL intensities in the same format as returned by trplLoad.
	%%
	%%			Returns:
	%%				data:	2D array of background subtracted TRPL intensities in the same format as returned by trplLoad.
	
	%% Identify pre-rise period
	prp_ind_start=1;
	prp_ind_end=round(0.05*size(data,1));										% TODO: Find a better way of calculating this
	
	%% Tidy up any very low values (e.g. from curvature correction)
	%data(find(data<0))=0;														% Should this be NaN or 0?
	
	%% Subtract average of pre-rise period at each wavelength
	data=data-repmat(median(data(prp_ind_start:prp_ind_end,:),1),size(data,1),1);
	
	%% Return
	return
	
end